#!/usr/bin/perl
#  Copyright 2001-2018 Leslie Richardson

#  This file is part of Open Admin for Schools.
#  Outline: View Evaluation Outline for Current School


my %lex = ('Error' => 'Error',
	   'Main' => 'Main',
	   'Goals' => 'Goals',
	   
	   'Strategies' => 'Strategies',
	   'Responsible' => 'Responsible',
	   'Long Term Objective' => 'Long Term Objective',
	   'Impact Area' => 'Impact Area',
	   'Please Log In' => 'Please Log In',
	   
	   );

my $self = 'evmstView.pl';

use DBI;
use CGI;
use CGI::Session;

eval require "ppp.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

eval require "$globdir/global.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

my $q = CGI->new;
my %arr = $q->Vars;

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);


# Get Session
my $session = new CGI::Session("driver:$dbtype;serializer:FreezeThaw",
 undef,{Handle => $dbh}) or die CGI::Session->errstr;

if ( not $session->param('logged_in') ){
    $userid = $session->param('userid');
    print $q->header;
    print qq{$doctype\n<html><head>\n};
    print qq{<link rel="stylesheet" href="$iepcss" type="text/css"></head><body>\n};
    print qq{[ <a href="$iephome">$lex{Main}</a> ]\n};
    print qq{<form action="login.pl" method="post" style="display:inline;">\n};
    print qq{<input type="hidden" name="userid" value="$userid">\n};
    print qq{<input type="hidden" name="script" value="$self">\n};
    print qq{<input type="submit" value="$lex{'Please Log In'}">\n};
    print qq{</form></body></html>\n};
    exit;
}

my $userid = $session->param('userid');
my $schooldbase = $session->param('schooldbase');

# Reset Login time
my $duration = $session->param('duration');
$session->expire('logged_in', $duration );

print $session->header;

# Get User Name
my $sth = $dbh->prepare("select lastname, firstname from educator where userid = ?");
$sth->execute( $userid );
if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
my ( $lastname, $firstname ) = $sth->fetchrow;

my $sdb = $dbh->quote( $schooldbase );
my $select = "and school = $sdb";


# Print Page Head
my $title = "View Student Goals";
print qq{$doctype\n<html><head><title>$title</title>\n};
print qq{<link rel="stylesheet" href="$iepcss" type="text/css">\n};
print qq{</head><body><a name="top"></a>[ <a href="$iephome">$lex{Main}</a> ]\n};

print qq{$firstname $lastname\n};
print qq{<h1>$title</h1>\n};


if ( not $arr{page} ) {
    showStartPage();

} elsif ( $arr{page} == 1 ) {
    delete $arr{page};
    showReport();
}


#-------------
sub showReport {
#-------------

    # foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

    # Load Configuration Vals for Impacts
    my $sth = $dbh->prepare("select id, datavalue from conf_system where dataname = ?");
    foreach my $field ( qw(g_ImpactArea g_Impact g_ImpactValue) ) {
	$sth->execute( $field );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	while ( my ($id, $datavalue) = $sth->fetchrow ) {
	    eval $datavalue;
	    if ( $@ ) {
		print "$lex{Error}: $@<br>\n";
		die "$lex{Error}: $@\n";
	    }
	}
    }

    
    # sort by student name;
    my (%sort, %name);
    my $sth = $dbh->prepare("select lastname, firstname from special where studnum = ?");
    foreach my $studnum ( keys %arr ) {
	$sth->execute( $studnum );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	my ($lastname, $firstname ) = $sth->fetchrow;
	$sort{"$lastname$firstname$studnum"} = $studnum;
	$name{$studnum} = "$firstname $lastname";
    }

    
    # Setup Queries
    # Get name of goal group (ie. category)

    my $sth = $dbh->prepare("select catnum from evmst_main where studnum = ?");
    
    my $sth1 = $dbh->prepare("select category from subject where catnum = ?");
    my $sth2 = $dbh->prepare("select * from evmst_main where catnum = ? and studnum = ?");
    
    my $sth3 = $dbh->prepare("select * from evmst_goal where mainid = ?");
    my $sth4 = $dbh->prepare("select * from evmst_staff where mainid = ?");
    my $sth5 = $dbh->prepare("select term, mastery from evmst_mastery where goalid = ? order by term");

    my %categorydesc; # global name of each category;  $catnum -> category (name)

    
    foreach my $key ( sort keys %sort ) {
	my $studnum = $sort{$key};


	my (%categorysort, %category);
	# Get his/her goal groups.
	$sth->execute( $studnum) ;
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	while ( my $catnum = $sth->fetchrow ) {
	    
	    $sth1->execute( $catnum ) ;
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	    my $category = $sth1->fetchrow;
	    
	    $categorysort{"$category$catnum"} = $catnum;
	    $categorydesc{$catnum} = $category;
	}

	# Print Student Name
	print qq{<h2>$name{$studnum}</h2><hr>\n};
	

	# Load impacts for this student
	my $sth6 = $dbh->prepare("select * from impact_stud where studnum = ?");
	my %imp;
	$sth6->execute( $studnum );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	while ( my $ref = $sth6->fetchrow_hashref ) {
	    my %r = %$ref;
	    if ( $r{impact} ) {
		$imp{$r{impact}} = $r{val};
	    } elsif ( $r{other} ) { # it must be an 'other' value.
		$imp{$r{other}} = $r{val};
	    } else { # an error;
		print qq{<h3>$lex{Error}: Empty Impact for $firstname $lastname ID:$r{id}</h3>\n};
	    }
	}

	# Display Impacts.
	my $firstimp = 1;
	foreach my $impact ( sort keys %imp ) {
	    if ( $firstimp ) {
		print qq{<table cellpadding="3" cellspacing="0" border="0">\n};
		print qq{<tr><th>Impacts</th><th>Support Level</th></tr>\n};
		$firstimp = 0;
	    }
	    print qq{<tr><td class="bra">$impact</td><td class="la">$g_ImpactValue{$imp{$impact}}</td></tr>\n};
	}
	if ( $firstimp == 0 ) { print qq{</table>\n}; }

	
	
	# populate team for this student
	my %team;
	my $sth = $dbh->prepare("select * from team where studnum = ?");
	$sth->execute( $studnum );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	while ( my $ref = $sth->fetchrow_hashref ) {
	    my %r = %$ref;
	    $team{$r{userid}}{name} = "$r{lastname},$r{firstname}";
	    $team{$r{userid}}{jobtitle} = $r{jobtitle};
	}

   
	# Loop through his/her Goal Groups
	my $first = 1;
	foreach my $catkey ( sort keys %categorysort ) {

	    my $catnum = $categorysort{$catkey};

	    # pull in evaluation master file.
	    $sth2->execute( $catnum, $studnum );
	    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	    my $evref = $sth2->fetchrow_hashref; 
	    my %evm = %$evref;
	    my $mainid = $evm{id};
   
	    print qq{<h2>$categorydesc{$catnum} ($catnum)</h2>\n};
	    print qq{<table cellpadding="4" cellspacing="0" border="1" style="margin-bottom:1em;">\n};
	    print qq{<tr><th colspan="2">Main</th></tr>\n};
   
	    # Main Overview (4 fields)
	    print qq{<tr><td class="bra">Target Skill</td>};
	    print qq{<td style="width:40em;">$evm{targetskill}</td></tr>\n};
	    
	    print qq{<tr><td class="bra">Long Term Objective</td>};
	    print qq{<td style="width:40em;">$evm{longtermobjective}</td></tr>\n};
	    
	    print qq{<tr><td class="bra">Current Peformance Level</td>};
	    print qq{<td style="width:40em;">$evm{currperform}</td></tr>\n};
	    
	    print qq{<tr><td class="bra">Evaluation Method</td>};
	    print qq{<td style="width:40em;">$evm{evalmethod}</td></tr>\n};
	    
	    print qq{</table>\n};


	    # Responsible People
	    print qq{<table cellpadding="3" cellspacing="0" border="1" style="margin-bottom:1em;">\n};
	    print qq{<tr><th>Staff</th><th>Role</th></tr>\n};

	    $sth4->execute( $mainid );
	    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	    while ( my $stref = $sth4->fetchrow_hashref ) {
		my %srec = %$stref;
		my $name;
		if ( $team{ $srec{userid} } ) { # we have a name
		    $name = $team{$srec{userid}}{name};
		} else { $name = $srec{userid}; }
		print qq{<tr><td style="vertical-align:top;">$name</td>};
		print qq{<td style="width:50em;">$srec{strategy}</td></tr>\n};
	    }
	    print qq{</table>\n};

   
	    # Print Goals (and associated Master Level, terms)
	    print qq{<table cellpadding="3" cellspacing="0" border="1" style="margin-bottom:2em;">\n};
	    print qq{<tr><th>Goal</th><th>Mastery</th></tr>\n};
    
	    $sth3->execute( $mainid );
	    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	    while ( my $gref = $sth3->fetchrow_hashref ) {
		my %gr = %$gref;  # goal master.
		my $goalid = $gr{id};

		print qq{<tr><td>$gr{description}</td><td>};
	
		# Now load the mastery records for this goal
		$sth5->execute( $goalid );
		if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
		while ( my ($term, $mastery) = $sth5->fetchrow ) {
		    print qq{Term $term - $mastery<br>\n};
		}
		print qq{</td></tr>\n};
	    }
	    print qq{</table>\n};


	} # Next Goal Group

    } # Next Student

    print qq{</table>[ <a href="#top">Top</a> ]</body></html>\n};

    exit;

} # end of showReport



#----------------
sub showStartPage {  # select the students
#----------------

    # Get students
    my $sth = $dbh->prepare("select lastname, firstname, studnum from special
      where school = ? order by lastname, firstname");
    $sth->execute($schooldbase);
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

    my $sth1 = $dbh->prepare("select count(*) from impact_stud where studnum = ?");
    my $sth2 = $dbh->prepare("select id from evmst_main where studnum = ?");
    my $sth3 = $dbh->prepare("select count(*) from evmst_goal where mainid = ?");
    
    
    # Start the form
    print qq{<form action="$self" method="post" style="display:inline;">\n};
    print qq{<input type="hidden" name="page" value="1">\n};

    # Start Table
    print qq{<h3>Select Students</h3>\n};
    print qq{<input type="submit" value="Continue">\n};
    print qq{<table cellpadding="5" cellspacing="0" border="1">\n};
    print qq{<tr><th>Student</th><th>Impacts</th><th>Goals</th></tr>\n};

    my $counter = 1;
   
    while ( my ($lastname, $firstname, $studnum) = $sth->fetchrow ) {

	$sth1->execute( $studnum );
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	my $count = $sth1->fetchrow;

	my $goalcount;
	$sth2->execute( $studnum );
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	while ( my $mainid = $sth2->fetchrow ) {
	    # count the number of goals in this goal group.
	    $sth3->execute( $mainid );
	    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	    my $gcount = $sth3->fetchrow; 
	    $goalcount += $gcount;
	}
		
	print qq{<tr><td>$counter.\n};
	print qq{<input type="checkbox" name="$studnum" value="1">\n};
	print qq{ $firstname $lastname ($studnum)</td>};
	print qq{<td class="cn">$count</td><td class="cn">$goalcount</td></tr>\n\n};
	$counter++;
    }


    print qq{</table>\n};
    print qq{<input type="submit" value="Continue">\n};
    print qq{</form></body></html>\n};
    exit;

}
