#!/usr/bin/perl
#  Copyright 2001-2017 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my %lex = ('Report' => 'Report',
	   'Goals Master' => 'Goals Master',
	   
	   'Note' => 'Note',
	   'View Log File' => 'View Log File',
	   'Main' => 'Main',
	   'Error' => 'Error',
	   'Gr' => 'Gr',
	   'Print/View'  => 'Print/View',
	   'Please Log In' => 'Please Log In',

	   );

$self = 'rptsubject.pl';

use DBI;
use CGI;
use CGI::Session;

my $q = new CGI;
my %arr = $q->Vars;

# Read config variables
eval require "ppp.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

eval require "liblatex.pl";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

my ($sec, $min, $hour, $mday, $mon, $year, $wday, 
 $yday, $iddst) = localtime(time);
$year = $year + 1900;
$mon++;
$wday++;
my $currdate = "$dow[$wday], $month[$mon] $mday, $year";

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

# Get Session
my $session = new CGI::Session("driver:$dbtype;serializer:FreezeThaw",
 undef,{Handle => $dbh}) or die CGI::Session->errstr;

if ( not $session->param('logged_in') ){
    $userid = $session->param('userid');
    print $q->header;
    print qq{$doctype\n<html><head>\n};
    print qq{<link rel="stylesheet" href="$iepcss" type="text/css"></head><body>\n};
    print qq{[ <a href="$iephome">$lex{Main}</a> ]\n};

    print qq{<form action="login.pl" method="post" style="display:inline;">\n};
    print qq{<input type="hidden" name="userid" value="$userid">\n};
    print qq{<input type="hidden" name="script" value="$self">\n};
    print qq{<input type="submit" value="$lex{'Please Log In'}">\n};
    print qq{</form></body></html>\n};
    exit;
}

my $userid = $session->param('userid');
my $schooldbase = $session->param('schooldbase');

# Reset Login time
my $duration = $session->param('duration');
$session->expire('logged_in', $duration );

print $session->header;

# Get User Name
my $sth = $dbh->prepare("select lastname, firstname from educator where userid = ?");
$sth->execute( $userid );
if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
my ( $lastname, $firstname ) = $sth->fetchrow;


my $sth = $dbh->prepare("select * from subject order by category, catnum, idnum");
$sth->execute;
if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr ; }


my $title = "Goals Master $lex{Report}";
print qq{$doctype\n<html><head><title>$title</title>\n};
print qq{<link rel="stylesheet" href="$iepcss" type="text/css">\n};
print qq{</head>\n};

print qq{<body>[ <a href="$iephome">$lex{Main}</a> ]\n};

print qq{<h1>$title</h1><h4>$currdate</h4>\n};


prPDFhead();

my $count;  # used to keep track to put in page break spaces.

while ( my $ref = $sth->fetchrow_hashref ){

    my %r = %$ref;

    $count++;

    %r = latex_filter( %r );

#    foreach my $key ( sort keys %r ) { print qq{K:$key V:$r{$key}<br>\n}; }
    
    $oldsub = $cursub;
    $cursub = $r{catnum};
  
    if ( $cursub ne $oldsub ) {
	if ( $notfirst ){ 
	    print TEX "\\end{tabular}\\bigskip\n\n"; 
	} else { 
	    $notfirst = 1;
	}
	print TEX "\\hfil \\section{\\large\\sf $r{category} ($r{catnum})}\\hfil\n\n";
	print TEX "\\begin{tabular}{p{1in}p{5.4in}p{1in}}\\hline\n";
    }

    print TEX " $r{catnum} - $r{idnum} & ";
    print TEX "$r{description} \\\\\n";

    if ( not ($count % 2) ) { # put in space for line splits
	print TEX "\\end{tabular}\n\n";
	print TEX "\\begin{tabular}{p{1in}p{5.4in}p{1in}}\\hline\n";
    }

    
} # End of While loop
print TEX "\\end{tabular}"; # complete table.

prPDFfoot(); # print rest of PDF web page.

print qq{</body></html>\n};



#------------
sub prPDFhead {
#------------
    $shortname = "subjrep$$";
    $fileName = "$shortname.tex";

    open(TEX,">$fileName") || die "Can't open tex file";

    print TEX "\\documentclass[12pt,letterpaper,oneside]{article}
\\usepackage{graphicx,array,newcent,colortbl,inputenc}
\\usepackage[pdftex,bookmarks,bookmarksopen,pdfstartview=FitH]{hyperref}
\\inputencoding{latin1}
\\setlength{\\textwidth}{7.50in}
\\setlength{\\textheight}{10in}
\\setlength{\\hoffset}{0in}
\\setlength{\\voffset}{-1in}
\\setlength{\\topmargin}{0.2in}
\\addtolength{\\evensidemargin}{-1in}
\\addtolength{\\oddsidemargin}{-1in}
\\setlength{\\extrarowheight}{2pt}
\\setlength{\\parindent}{0in}
\\pagestyle{myheadings}
\\markright{$divisionname \\hfill $lex{'Goals Master'} $lex{Report} \\hfill $currdate -- Pg }
\\begin{document}\n";

}


#------------
sub prPDFfoot {
#------------

    print TEX "\\end{document}";
    close TEX;
    system("$pdflatex $fileName > pdflog$$.txt");
    system("$pdflatex $fileName > /dev/null");
    #system("$pdflatex $fileName > /dev/null");
    # Compile 3 times to make sure all links built right.
    system("mv $shortname.pdf $downloaddir");
    system("mv pdflog$$.txt $downloaddir");
    system("rm -f $shortname.*");

    print qq{<h2><a href="$webdownloaddir/$shortname.pdf">};
    print qq{$lex{'Print/View'} $lex{Report}</a></h2>\n};
    print qq{<p>[ <a href="$iephome">$lex{Main}</a> | \n};
    print qq{<a href="$webdownloaddir/pdflog$$.txt">};
    print qq{$lex{'View Log File'}</a> ] </p>\n};

} # end of pfPDFfoot
