#! /usr/bin/perl
#  Copyright 2001-2010 Leslie Richardson

#  This file is part of Open Admin for Schools.
#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

# This script will update Sask Ed with changes to student record fields.
#  which are located in the utag fields of the student table.

use DBI;
use CGI;
use XML::Writer;
use XML::Writer::String;
use Data::UUID;
use HTTP::Request::Common qw(POST); 
use HTTP::Headers; 
use LWP::UserAgent; 
use XML::LibXML;
use Number::Format qw(round);

my $self = 'updatedemog.pl';

# Used for Filename Date
my @smonth = ('', 'JAN','FEB','MAR','APR','MAY','JUN','JUL','AUG','SEP','OCT','NOV','DEC');

my %lex = ( 'Main' => 'Main',
	    'Error' => 'Error',
	    'View/Download' => 'View/Download',
	    'Enrollment File' => 'Enrollment File',

	    );

eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

require "slxmllibNew.pl";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}


my $q = new CGI;
print $q->header( -charset, $charset);
my %arr = $q->Vars;


print "$doctype\n<html><head><title>SDS Demographics Update</title> 
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
$chartype\n</head><body>[ <a href=\"$homepage\">Main</a> | 
<a href=\"$exppage\">Export</a> ]<h1>Student Demographics Update</h1>\n";

if ( not $arr{page} ) {
    showStartPage();
} else {
    delete $arr{page};
}


if ($arr{debug}){
    if ( not $arr{flag} ) { # Show the options for debug data errors.
	showDebug();
    } else { # Set the data error options
	$maxfilesize = $arr{xmlsize}; # override values in config
    }
}

my $filemode = $arr{filemode};
delete $arr{filemode};
if ( $filemode ) { $maxfilesize = '1000000'; } 


# Setup Date and Time
# $sec, $min, $hour, $mday, $mon, 
# $year, $wday, $yday, $isdst ) = localtime(time);
my @tim = localtime(time);
my $year = $tim[5] + 1900;
$tim[4]++;
# For Filename
my $fyear = $year - 2000;
my $filecurrdate = $tim[3]. $smonth[ $tim[4] ]. $fyear;
my $filecurrtime = $tim[2]. $tim[1];
# End of filename specific time
for (0..4){if (length($tim[$_]) == 1){ $tim[$_] = '0'.$tim[$_];}}
my $currdate = "$year-$tim[4]-$tim[3]";
my $currtime = "$tim[2]:$tim[1]:$tim[0]";

my $dsn = "DBI:$dbtype:dbname=$dbase";
$dbh = DBI->connect($dsn,$user,$password);
$dbh->{mysql_enable_utf8} = 1;

# Find students by lastname, firstname
my $sth = $dbh->prepare("select * from student 
 order by lastname, firstname");
$sth->execute;
my $rows = $sth->rows;
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

# Create a new user agent
my $ua = LWP::UserAgent->new();
$ua->agent("OpenAdmin");


my $donecount = 1;
my $reccount = $rows;
my $xfercount = 1;

my $filename;
if ( $filemode ) { # open the file for writing.
    $filename = 'ENROL_'. $schoolnumber. '_'. $filecurrdate. '_'. $filecurrtime. '.xml';
    open(FH,">$filename") or die "Cannot Open XML file\n";
}



while ( $donecount <= $reccount ) {

    # Create Writer Instance
    my $output = XML::Writer::String->new();
    my $datamode = 0;
    if ($arr{debug}){ $datamode = 1;} # pretty print xml output
    $wr = new XML::Writer(OUTPUT => $output, 
			  DATA_MODE => $datamode, 
			  DATA_INDENT => '2');

    # Set XML Header and write Root Element
    $wr->xmlDecl("UTF-8");
    $wr->startTag('SL_Message','xmlns' =>$xmlns, 
		  'xmlns:xsi' =>$xmlnsxsi, 
		  'xsi:schemaLocation' => $xsischemaLocation);
    
    $wr->startTag('SL_Event');

    $srcMsgId = mkSL_Header($currdate,$currtime, $schoolnumber,$xfercount);

    $xfercount++; # update counter to keep header unique

    $wr->startTag('SL_ObjectData');

    print "<span style=\"color:black;font-size:150%;\">Query</span><br>\n";
    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
    print "<tr><th>Student Name</th></tr>\n";
    
    my $masternameflag = 0;

    while ( my $ref = $sth->fetchrow_hashref ) {

	my %sr = (); # empty it out...
	%sr = %$ref;

	if ( not $sr{provnum} ){ # No Prov Number
	    print "<tr><td>$donecount:";
	    print "<a href=\"$cgiurl/studed.pl?id=$sr[0]\">";
	    print "<b>$sr{lastname}</b>, $sr{firstname}</a> ($sr{studnum})</td>\n<td>";
	    print "<span style=\"color:red;\">No Provincial Number!</span>";
	    print "</td></tr>\n";
	    $donecount++;
	    next;
	}


	#if ($ufld == 72 ){ # create program enrollment
	# $wr->startTag('SL_EventObject',
	#                    'ObjectName' => 'StudentProgramEnrollment',
	#		      'Action' => 'Change');
	#       mkStudentProgramEnrollment( $sr[57],$sr[8],$sr[72],$currdate ); 
	#	# function above puts a .1 after studnum in refid.
	#	$wr->endTag('SL_EventObject');
        # }
	  
	# Change Grade Object
	$wr->startTag('SL_EventObject',
		      'ObjectName' => 'StudentSchoolEnrollment',
		      'Action' => 'Change');

	# Build StudentEnrollmentInfo (sei) Hash
	$seiRef = {'MembershipType' => '', 
		   'EntryDate' => '', 'Grade' => "$sr{grade}",
		   'PrevProv' => '', 'PrevCountry' => '',
		   'ExitType' => '', 'ExitDate' => '',
		   'ImmersionType' => $sr{immersion_type},
		   'TuitionType' => $sr{tuition_status},
		   'TuitionLength' => $sr{tuition_duration},
		   'TuitionCollected' => $sr{tuition_collect},
		   'TuitionExchangeProgram' => $sr{tuition_program}
	       };

	#print "Immersion Type: $sr{immersion_type}<br>\n";

	$refid = $sr{studnum}.".1"; # Student with a .1 to make unique #
	$wr->startTag('StudentSchoolEnrollment', RefId=>$refid);
	mkStudentIdentification( $sr{provnum}, $sr{birthdate} ); # provnum/bdate 


	$wr->dataElement('SchoolId',$schoolnumber); # from admin.conf
	
	mkSchoolEnrollmentInfo($seiRef);

	$wr->endTag('StudentSchoolEnrollment');

	$wr->endTag('SL_EventObject');


	my $studentname = "<b>$sr{lastname}</b>, $sr{firstname}";
	print "<tr><td>$donecount. ";
	print "<a href=\"$cgiurl/studed.pl?id=$sr{studid}\">$studentname</a>";
	print " ($sr{studnum})</td></tr>\n";
	$donecount++;


	$wr->startTag('SL_EventObject',
		      'ObjectName' => 'StudentPersonal',
		      'Action' => 'Change');

	$refid = $sr{studnum}; # Student #
	$studentupdated{$refid} = 1; # Hash for updated records.

	$wr->startTag('StudentPersonal', RefId=>$refid);
	mkStudentIdentification( $sr{provnum},$sr{birthdate}); # parse provnum and bdate 

	#--------------Main Entry Approach --------------------------
	mkStudentInfo( $ref );
	#--------- Line Above crucial to entire process.---------

	$wr->endTag('StudentPersonal');
	$wr->endTag('SL_EventObject');

	$filesize = length($output->value);
	if ( $filesize >= $maxfilesize ){ last; } # break out of loop 

    } # End of Student Loop.

    $wr->endTag('SL_ObjectData');
    $wr->endTag('SL_Event');
    $wr->endTag('SL_Message');
    $wr->end();

    print "</table>\n";


    # Write all data to batch file.
    if ( $filemode ) {
	my $temp = $output->value;
	print FH $temp;
    }


    # DEBUG Data Errors
    if ( $arr{sentdata} ){
	print "<div style=\"color: blue;\">\n";
	print "<h1>DEBUG - Request sent to Sask Ed</h1>\n";
	my $temp = $output->value;
	$temp =~ s/</&lt;/g;
	$temp =~ s/>/&gt;/g;
	print "<pre>",$temp,"</pre></div><p></p>\n";
    }


    if ( not $filemode ) {

	my $filesize = length( $output->value ) / 1000 ;
	print "The filesize is ", round( $filesize, 1 ), "KB (Limit: 32KB)\n";


	# Create the https post request 
	my $req = POST $url, [ XML=>$output->value ];

	$req->content_type('application/xml;charset="utf-8"');
	$req->authorization_basic($sds_userid, $sds_password); 

	# Issue the request and receive a response
	my $res = $ua->request($req);

	# Check the status of the response
	if ( $res->is_success ) {

	    # For Debugging Data Errors
	    if ( $arr{recddata} ) {
		print "<h1>DEBUG - Sask Ed XML Response</h1>\n";
		print "<div style=\"color:green;\">\n";
		my $temp = $res->content;
		$temp =~ s/</&lt;/g;
		$temp =~ s/>/&gt;/g;
		print "<pre>",$temp,"</pre></div>\n";
	    }

	    print "<p><b>Results of Request to Sask Ed</b></p>\n";

	    # Parse the response.
	    my $parser = XML::LibXML->new();

	    eval { $doc = $parser->parse_string($res->content) };
	    if ( $@ ) {
		print "Sask Ed Error: <pre>",$res->content,"</pre><br>\n";
		print "</body></html>\n"; die;
	    }

	    $doc->setEncoding('UTF-8');
    
	    my $root = $doc->getDocumentElement;
	    $root->setNamespace($xmlns,'sl',1);

	    $mastermsgid = $root->findvalue('//sl:SL_MsgId');
	    #print "<p><b>MsgId:</b> $mastermsgid</p>\n";

	    my @errors = $root->findnodes('//sl:SL_Error');
	    if ( @errors ) { 
		print "<span style=\"color:black;font-size:150%;\">Results</span>\n";
		print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
		print "<tr><th>Student</th><th>Response</th></tr>\n";

		# prepare for getting name in the 'error' response loop.
		$sth1 = $dbh->prepare("select studid, lastname,firstname 
                  from student where studnum = ?");

		foreach my $error ( @errors ) {
		    my $studentrefid = $error->getAttribute('RefId');
		    $studentrefid =~ s/\..*//; # strip the .1
		    my $errcode = $error->findvalue('sl:SL_ErrorCode');

		    # Get the Name
		    $sth1->execute( $studentrefid );
		    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
		    my ($studid,$lastname, $firstname) = $sth1->fetchrow;
		
		    my $objectname = $error->getAttribute('ObjectName');
		    my $object;
		    if ( $objectname eq 'StudentPersonal' ) {
			$object = 'Demographics Update';
		    } else {
			$object = 'Grade Update';
		    }


		    my $errmsg = $error->findvalue('./sl:SL_ErrorMsg');
		    if ( $errmsg ne 'Transaction Successful' ) {
			$errmsg = "<span style=\"color:red;font-weight:bold;\">$errmsg</span>";
		    }
		
		    print"<tr><td><a href=\"$cgiurl/studed.pl?id=$studid\">";
		    print "$lastname, $firstname</a> ($studentrefid)</td>";
		    print "<td><b>$object:</b> $errmsg</td></tr>\n";
	    
		} # End of For Errors

		print "</table>\n";

	    } # End of IF Errors
	
	} else { # We have a transfer error!
	    my $err = $res->status_line;
	    print "<h1>Transfer Error: $err</h1>";
	    exit;
	}
	print "<p><hr></p>\n";

    } # end of not $filemode

} # End of Counted Loop for all students.


if ( $filemode ) { # Filemode here (ie. file upload )

    close FH;

    # File download stuff here.
    system("mv $filename $downloaddir");

    print "<h1>". $lex{'View/Download'}. ' '. $lex{'Enrollment File'};
    print "</h1>\n<p>[ <a href=\"$webdownloaddir/$filename\">";
    print "$filename</a> ]</p>\n";
	
} 

print "</body></html>\n";



#----------------
sub showStartPage {  # Show Options.
#----------------

    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";

    # Batch File / Real Time setting.
    print "<div style=\"width:40%;border:1px solid black;padding:0.4em;text-align:center;font-size:130%;\">\n";
    print "Create <b>Batch File</b> for Upload to Sask Ed ? \n";
    print "<input type=\"checkbox\" name=\"filemode\" value=\"1\">\n";
    print "<br>No Real Time Transfers will be done.</div>\n";

    print "<p><b>Debug Transfers?</b> <input type=\"checkbox\" name=\"debug\" value=\"1\"></p>\n";
    print "<input type=\"submit\" value=\"Continue\"></form>\n";

    print "</body></html>\n";

    exit;

}



#------------
sub showDebug {  # Show debug options...
#------------

    print "<h1>Demographics Update -- Debug Options</h1>\n";

    print "<p style=\"width:40%;border:1px solid gray;padding:0.4em;\">\n";
    print "In order to find problems with submitted data, the sizes of the XML \n";
    print "files (and the number of contained records) should be limited so that\n";
    print "it is easier to pick out the offending data. If size is less than 3000\n";
    print "characters, this will limit data to a couple of student records (which\n";
    print "are relatively easy to locate data errors within).</p>\n";

    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"flag\" value=\"1\">\n";
    print "<input type=\"hidden\" name=\"debug\" value=\"1\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";

    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\">\n";

    print "<tr><td></td><td><input type=\"submit\" value=\"Continue\"></td></tr>\n";

    print "<tr><td align=\"right\">XML File Size Limit:</td>\n";
    print "<td><input type=\"text\" name=\"xmlsize\" size=\"8\" value=\"3000\"></td></tr>\n";

    print "<tr><td align=\"right\">View Submitted Data</td>";
    print "<td><input type=\"checkbox\" name=\"sentdata\" value=\"1\" checked></td></tr>\n";

    print "<tr><td align=\"right\">View Received Data</td>\n";
    print "<td><input type=\"checkbox\" name=\"recddata\" value=\"1\" checked></td></tr>\n";

    print "<tr><td></td><td><input type=\"submit\" value=\"Continue\"></td></tr>\n";
    print "</table></body></html>\n";

    exit;

}
