#!/usr/bin/perl
#  Copyright 2001-2016 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my %lex = ('Error' => 'Error',
	   'Student Evaluation' => 'Student Evaluation',
	   'Main' => 'Main',
	   'School' => 'School',
	   'Term' => 'Term',
	   'Student Evaluation Entry' => 'Student Evaluation Entry',
	   'Grade' => 'Grade',
	   'Add/Edit' => 'Add/Edit',
	   'No Records Found' => 'No Records Found',
	   'Term to Add/Edit' => 'Term to Add/Edit',
	   'Other Student Evaluations' => 'Other Student Evaluations',
	   'Edit More' => 'Edit More',
	   'Please record the following error' => 'Please record the following error',
	   'Contact' => 'Contact',
	   'There was an error storing your data' => 'There was an error storing your data',
	   'The evaluation record is now stored/updated' => 
	     'The evaluation record is now stored/updated',
	   'Subject not found!' => 'Subject not found!',
	   'Objectives' => 'Objectives',
	   'Eval' => 'Eval',
	   'Comment' => 'Comment',
	   'Continue' => 'Continue',
	   'Please Log In' => 'Please Log In',

	   );

my $self = 'iepevaladd.pl';

use DBI;
use CGI;
use CGI::Session;

my $q = new CGI;
my %arr = $q->Vars;
my $studnum = $arr{studnum};

my @tim = localtime(time);
my $year = $tim[5] + 1900;
my $month = $tim[4] + 1;
my $currdate = "$year-$month-$tim[3]";


eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

# Get Session
my $session = new CGI::Session("driver:$dbtype;serializer:FreezeThaw",
 undef,{Handle => $dbh}) or die CGI::Session->errstr;

if ( not $session->param('logged_in') ){
    $userid = $session->param('userid');

    print $q->header;
    print "$doctype\n<html><head>\n";
    print "<link rel=\"stylesheet\" href=\"$tchcss\" type=\"text/css\"></head><body>\n";
    print "[ <a href=\"$tchpage\">$lex{Main}</a> ]\n";
    print "<form action=\"../tlogin.pl\" method=\"post\" style=\"display:inline;\">\n";
    print "<input type=\"hidden\" name=\"userid\" value=\"$userid\">\n";
    print "<input type=\"hidden\" name=\"script\" value=\"iep/$self\">\n";
    print "<input type=\"submit\" value=\"$lex{'Please Log In'}\">\n";
    print "</form></body></html>\n";
    exit;

}

my $userid = $session->param('userid');

# Reset Login time
my $duration = $session->param('duration');
$session->expire('logged_in', $duration );

print $session->header;

# Get User Name
my $sth = $dbh->prepare("select lastname, firstname from staff where userid = ?");
$sth->execute( $userid );
if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
my ( $lastname, $firstname ) = $sth->fetchrow;

# Homerooms and Access for this teacher.
my (%homerooms, %access );
if ( $arr{page} == 2 ) { # get access checking ready

    # Get homeroom and check access credentials.
    my $sth = $dbh->prepare("select field_value from staff_multi where field_name = 'homeroom' and userid = ?");
    $sth->execute( $userid );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    while ( my $homeroom = $sth->fetchrow ) {
	$homerooms{$homeroom} = 1;
    }

    # Get Access credentials
    my $sth = $dbh->prepare("select field_value from staff_multi where field_name = 'access' and userid = ?");
    $sth->execute( $userid );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    while ( my $ac = $sth->fetchrow ) {
	$access{$ac} = 1;
    }
}


# Print Page Heading
my $title = "$lex{Student} $lex{Evaluation}";
print "$doctype\n<html><head><title>$title</title>\n";
print "<link rel=\"stylesheet\" href=\"$tchcss\" type=\"text/css\"></head>\n";
print "<body>[ <a href=\"$tchpage\">$lex{Main}</a> ] $firstname $lastname\n";


# Disable entry if all tracks set to zero entry term.
my $disableentry = 1; # stop entry by default.
my $maxterm; # used below to show how many terms to count up to for entry.
foreach my $trk ( sort keys %g_MTrackTerm ){
    if ( $r_MarkEntryTerm{$trk} ) { $disableentry = 0; } # any nonzero value to enable mark entry.
    foreach my $trm ( keys %{$g_MTrackTerm{$trk}} ) {  
	if ( $trm > $maxterm ) { $maxterm = $trm; } # set maxterm value;
    }
}

if ( $disableentry ) { # only if all tracks are zero
    print qq{<h3>$lex{'System Disabled. Please contact secretary'}</h3></body></html>\n};
    exit;
}



# Decide on what function to call.
if ( not $arr{page} ) {
    showStartPage();

} elsif ( $arr{page} == 1 ) {
    delete $arr{page};
    selectSubject();

} elsif ( $arr{page} == 2 ) {
    delete $arr{page};
    addEditRecord();

} elsif ( $arr{page} == 3 ) {
    delete $arr{page};
    writeRecord();
}


#----------------
sub showStartPage {
#----------------

    my $sth = $dbh->prepare("select * from iep_student
      order by lastname, firstname");
    $sth->execute;
    if ( $DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

    # print "<div style=\"padding:0.5em;\">$lex{Term}: $term</div>\n";

    print "<table border=\"1\" cellpadding=\"3\" cellspacing=\"0\">\n";
    print qq{<tr><th>Name</th><th>Grade</th><th>Subjects</th><th>Edit</th></tr>\n};

    while ( my $ref =  $sth->fetchrow_hashref ) {
	my %r = %$ref;
	my $studnum = $r{studnum};

	# Get Grade, Homeroom from studentall table.
	my $sth1 = $dbh->prepare("select grade, homeroom from studentall where studnum = ?");
	$sth1->execute($studnum);
	if ( $DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	my ($grade, $homeroom) = $sth1->fetchrow;

	# Check for access, including a course teacher
	my $accessflag = 1;
	if ( not $access{IEP} and not $homerooms{$homeroom} ){

	    # Check if any grades have subject attendance, 
	    # and check those grades for course enrollments
	    my $sth = $dbh->prepare("select count(e.id) from eval e, subject s 
             where e.studnum = ? and e.subjcode = s.subjsec and s.teacher = ?");
	    $sth->execute( $studnum, $userid );
	    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }  
	    my $count = $sth->fetchrow;
	    if ( $count < 1 ) {
		#print qq{<h3>No Access Allowed</h3>\n};
		#print qq{</body></html>\n};
		$accessflag = 0;
		# exit;
	    }
	}

	# Count Subjects (up to 32 objectives per subject)
	my $sth = $dbh->prepare("select count(*) from iep_evalmst where studnum = ?");
	$sth->execute( $studnum );
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }  
	my $subjcount = $sth->fetchrow;


	print "<tr><td><b>$r{lastname}</b>, $r{firstname}</td><td>$lex{Grade}: $r{grade}</td>\n";
	print qq{<td class="cn">$subjcount</td>\n};
	if ( not $accessflag ) {
	    print "<td><form action=\"$self\" method=\"post\">\n";
	    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";
	    print "<input type=\"hidden\" name=\"studnum\" value=\"$r{studnum}\">\n";
	    print qq{<input type="submit"  value="$lex{'Add/Edit'}"></form></td>\n};

	} else {
	    print qq{<td>No Access</td>\n};
	}
	print qq{</tr>\n};
    }

    print "</table></body></html>\n";

    exit;

} # end of showStartPage


#----------------
sub selectSubject {
#----------------

    my $studnum = $arr{studnum};

    my $sth = $dbh->prepare("select lastname, firstname, grade from iep_student where studnum = ?");
    $sth->execute( $studnum );
    if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }
    my ( $lastname, $firstname, $grade ) = $sth->fetchrow;

    # Print Form Start
    print "<h2>$firstname $lastname</h2>\n";
    print "<table border=\"1\" cellspacing=\"0\" cellpadding=\"3\">\n";
    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"2\">\n";
    print "<input type=\"hidden\" name=\"studnum\" value=\"$studnum\">\n";


    # Print the Term Selection
    print qq{<tr><td><b>$lex{'Term to Add/Edit'}</b>\n};
    print qq{ <select name="term"><option>$term</option>\n};
    for my $trm ( 1..$maxterm ) { # maxterm defined in start section.
	print qq{<option>$trm</option>};
    }
    print qq{</select></td></tr>\n};


    my $firstflag = 1;

    # Get Objective Info for this student
    my $sth1 = $dbh->prepare("select distinct category, catnum 
      from iep_objectives o, iep_evalmst e where e.studnum = ? and
      e.subnum = o.catnum  order by o.category");
    $sth1->execute( $studnum );
    if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }

    # Now loop through his/her subjects; pick one to edit
    while ( my ( $category, $catnum ) = $sth1->fetchrow ) {
	print "<tr><td><input type=\"submit\" name=\"$catnum\" ";
	print "value=\"$category ($catnum)\"></td></tr>\n";
	$firstflag = 0;
    }

    print "</table></form>\n";

    if ( $firstflag ) {
	print "<p><b>$lex{'No Records Found'}</b></p>\n";
    }
    print "</body></html>\n";

    exit;

} # end of selectSubject



#----------------
sub addEditRecord {
#----------------

    # foreach my $key ( sort keys %arr ) { print qq{K:$key V:$arr{$key}<br>\n}; }

    # Get studnum, term, and subject
    my $studnum = $arr{studnum};
    delete $arr{studnum};
    my $term = $arr{term};
    delete $arr{term};

    my ($catnum, $dud) = each %arr; # since only one left in %arr hash.

    # get student name
    my $sth = $dbh->prepare("select lastname, firstname, grade from iep_student where studnum = ?");
    $sth->execute( $studnum );
    if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }
    my ( $lastname, $firstname, $grade ) = $sth->fetchrow;
    

    # Print Heading 
    print "<h3>$firstname $lastname - $lex{Term} $term</h3>\n";

    # Get Subject Name
    $sth = $dbh->prepare("select category from iep_objectives where catnum = ?");
    $sth->execute( $catnum );
    if ( $DBI::errstr){print $DBI::errstr; die $DBI::errstr; }
    my $category = $sth->fetchrow;


    # Get Subject Info from evalmst for this student
    my $sth1 = $dbh->prepare("select * from iep_evalmst where studnum = ? and subnum = ?");
    $sth1->execute( $studnum, $catnum );
    if ($DBI::errstr){print $DBI::errstr; die $DBI::errstr; }

    # Test Code. Should never run.
    my $rows = $sth1->rows;
    if ($rows < 1){
	print "</table></form><h1>$lex{'Subject not found!'}</h1>Studnum:$studnum Catnum:$catnum</body></html>\n";
	die $lex{'Subject not found!'}. "\n";
    }


    my $emref = $sth1->fetchrow_hashref;
    my %em = %$emref;


    # Read in evaluation record (if it exists) from eval.
    my $sth3 = $dbh->prepare("select * from iep_eval where studnum = ? and subnum = ? and term = ?");
    $sth3->execute( $studnum, $catnum, $term );
    if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }

    my $testrows = $sth3->rows;
    if ($testrows > 1){ # We have an error!
	print "<h1>Error! More than 1 evaluation record for $studentname";
	print "for $category for term $term. Please contact Les Richardson.";
	die "Multiple Evaluation Records!\n";
    }

    my $ref = $sth3->fetchrow_hashref;
    %eval = %$ref;
    my $evalid;
    if ( $testrows == 1 ) { $evalid = $eval{id};} # pass whether add or edit mode.


    # Print Form section
    print"<h2>$category ($catnum)</h2>\n";
    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name = \"page\" value = \"3\">\n";
    print "<input type=\"hidden\" name = \"studnum\" value = \"$studnum\">\n";
    print "<input type=\"hidden\" name = \"term\" value = \"$term\">\n";
    print "<input type=\"hidden\" name = \"subnum\" value = \"$catnum\">\n";
    print "<input type=\"hidden\" name=\"id\" value=\"$evalid\">\n";
    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";

    print qq{<tr><th>$lex{Eval}</th><th>$lex{Objectives}</th></tr>\n};

    for my $i (1..32) {
	my $key = 'obj'. $i;
	if ( $em{$key} ) {
	    print qq{<tr><td><input type="text" name="$key" size="6" value="$eval{$key}"></td>\n};
	    print qq{<td>$em{$key}</td></tr>\n};
	}
    } # End For loop

    print qq{<tr><td colspan="2" class="bla">$lex{Comment}<br>\n};
    print qq{<textarea name="comment" cols="100" rows="4">$eval{comment}</textarea>\n};
    print qq{</td></tr>\n};

    print qq{</table><input type="submit" value="$lex{Continue}">\n};
    print qq{</form></body></html>\n};

    exit;

}


#----------------
sub writeRecord {
#----------------

    # foreach my $key ( sort keys %arr ) { print qq{K:$key V:$arr{$key}<br>\n}; }

    # Get Student Name
    $sth = $dbh->prepare("select lastname,firstname from iep_student where studnum = ?");
    $sth->execute( $arr{studnum} );
    if ($DBI::errstr){print $DBI::errstr; die $DBI::errstr; }
    my ($lastname,$firstname) = $sth->fetchrow;


    my $studentname = "$firstname $lastname";  
    my $studnum = $arr{studnum}; # non quoted version needed for link below.

    # ADD Mode
    if ( not $arr{id} ){ 

        delete $arr{id}; # delete hash key if no value found.

	my @keys = keys %arr;
	my $keys = join(',', @keys);
	
	foreach my $k ( keys %arr ) {
	    $arr{$k} = $dbh->quote( $arr{$k} );
	}
	my $values = join(',', ( values %arr ));

	$sth = $dbh->prepare("insert into iep_eval ( $keys ) values ( $values )");
	$sth->execute;

    }  else {  # UPDATE Mode

	my $id = $arr{id}; # remove the record id from values to be updated.
	delete $arr{id}; 

	foreach my $key ( keys %arr ) {
	    my $sth = $dbh->prepare("update iep_eval set $key = ? where id = $id ");
	    $sth->execute( $arr{$key} );
	}

    } # End of Update Mode.

    if ( not $DBI::errstr ) {
	print "<h3>". $lex{'The evaluation record is now stored/updated'}. ".</h3></p>";

    } else {
	print "<h3>". $lex{'There was an error storing your data'}. ". \n";
	print $lex{'Contact'}. " $adminname <a href=\"mailto:$adminemail\">$adminemail</a>\n";
        print $lex{'Please record the following error'}. ": $DBI::errstr </h3>\n";
    }

    print "<p>[ <a href=\"$tchpage\">$lex{Main}</a> |\n";

    print "<a href=\"$self\">$lex{'Other Student Evaluations'}</a> |\n";
    print "<a href=\"$self?page=1&studnum=$studnum\">";
    print "$lex{'Edit More'} $studentname</a> ]\n";

    # Method for going one page deeper, but no term select... remove
    # Get Subject Info for this student
    #$sth = $dbh->prepare("select distinct subject.category, subject.catnum 
    # from subject,evalmst where evalmst.studnum = ? and 
    # evalmst.subnum = subject.catnum order by subject.category");
    #$sth->execute( $studnum );
    #if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }

    #while ( my ($category,$catnum) = $sth->fetchrow ) {
    #print "<tr><td><a href=\"$self?choice=$catnum&studnum=$arr{studnum}\">";
    #print "$category ($catnum)</a></td></tr>\n";
    # }
    #print "</table>";

    print "</body></html>\n";

    exit;

} # end of writeRecord
