#!/usr/bin/perl
#  Copyright 2001-2010 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

# Dual Run Script:  cgi and tcgi

%lex = ('Withdrawn Students' => 'Withdrawn Students',
	'Main' => 'Main',
	'Top' => 'Top',
	'No Records Found' => 'No Records Found',
	'All Records' => 'All Records',
	'Error' => 'Error',
	'Continue' => 'Continue',
	'Lastname' => 'Lastname',
	'Birthdate' => 'Birthdate',
	'Grade' => 'Grade',
	'Homeroom' => 'Homeroom',
	'Sort by' => 'Sort by',
	'Select by' => 'Select by',
	'View' => 'View',
	'Students' => 'Students',

);

use DBI;
use CGI;
use Cwd;

# Read config variables
eval require "../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

eval  require "../lib/libmeta.pl";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

my $q = new CGI;
print $q->header( -charset, $charset );
my %arr = $q->Vars;

my $mode;  # normal or withdrawn mode
my $table = 'student';
if ( $arr{table} eq 'wd' ) {
    $mode = 'wd';
    $table = 'studentwd';
}


my ($sec, $min, $hour, $mday, $mon, $year, $wday, 
    $yday, $iddst) = localtime(time);
$year = $year + 1900;
$mon++;
$wday++;
my $currdate = "$dow[$wday], $month[$mon] $mday, $year";


# Get current dir so know what CSS to display;
if ( getcwd() =~ /tcgi/ ){ # we are in tcgi
    $css = $tchcss;
    $homepage = $tchpage;
}


# Display top of page
print "$doctype\n<html><head><title>". $lex{View}. q{ }. $lex{Students}. "</title>
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">\n";
if ($mode eq 'wd') { 
    print "<style type=\"text/css\">th { background-color:#707;} h1 { color:#707; }</style>\n";
}

print "$chartype\n</head><body><a name=\"top\"></a>\n";
print "[ <a href=\"$homepage\">". $lex{Main}. "</a> ]\n";

# Show start page if only beginning.
if ( not $arr{page} ) {
    showStartPage();
} else {
    delete $arr{page};
}


my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);
$dbh->{mysql_enable_utf8} = 1;
$dbh->{mysql_enable_utf8} = 1;

# Passed: groupType, groupValue, sortorder.
my ($group, $groupDisplay, $select);
if ( $arr{groupType} eq $lex{Grade} ) {
    $group = 'grade';
} else {
    $group = 'homeroom';
}
if ( $arr{groupValue} ) {
    my $grpval = $dbh->quote( $arr{groupValue} );
    $select = "where $group = $grpval";
    $groupDisplay = ucfirst( $arr{groupType} ). " $arr{groupValue}";
} else { # no select; reset groupValue
    $groupDisplay = $lex{'All Records'};
} 

my $sortorder = "lastname, firstname";
if ( $arr{sortorder} eq $lex{Birthdate} ) {
    $sortorder = "birthdate";
}


print "<h1>";
if ($mode eq 'wd') { 
    print $lex{View}. q{ }. $lex{'Withdrawn Students'}; 
} else {
  print $lex{View}. q{ }. $lex{Students};
}

print " - $groupDisplay</h1><h3>$currdate</h3>\n";


# Read in Template
eval open (FH,"<../template/student.tpl");
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

my $formtext;
{ local $/; $formtext = <FH>; close FH;}

# Create hash for fieldnames from meta.
my $sth = $dbh->prepare("select fieldid, fieldname, required from meta where tableid = ?");
$sth->execute( student );
if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
my %fieldnames = ();
while ( my ( $fieldid, $fieldname, $required ) = $sth->fetchrow ) {
    if ( $required ) { # has any value
	$fieldname = "<span style=\"font-weight:bold;\">$fieldname</span>";
    }
    $fieldnames{$fieldid} = $fieldname;
}

# Now put replacement text back in.
$formtext =~ s{\<\*(.*?)\*\>}
  { exists( $fieldnames{$1} ) 
	? $fieldnames{$1} 
	: $1
  }gsex;
#print $formtext;


# OLD Code
#my $sth1 = $dbh->prepare("select fieldid,arrayidx from meta where
#  tableid = 'student'");
#$sth1->execute;
#if ($DBI::errstr){ print $DBI::errstr; die;}
#while (my ($fieldid,$arrayidx) = $sth1->fetchrow){
#    $values{$fieldid} = '$arr['.$arrayidx.']';
#}

# Now put replacement text back in.
#$formtext =~ s{\<\#(.*?)\#\>}
#          { exists($values{$1}) 
#		  ? $values{$1} 
#	          : $1
#          }gsex;


#$result = 'print <<"EOF";'."\n".$formtext."\n".'EOF';

my $sth = $dbh->prepare("select * from $table $select order by $sortorder");
$sth->execute;
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }

my $first = 1;

while ( my $ref = $sth->fetchrow_hashref ){

    my %rec = %$ref;
    my $text = $formtext; # make a copy to use.

    # now put field values back into $text variable...
    $text =~ s{ \<\@(.*?)\@\> }
    { exists($rec{$1}) 
	? $rec{$1} 
	: "$rec{$1}-$1"
    }gsex;

    print "<hr><div style=\"background-color:#EEE;border 1px solid black;padding:1em;\">";
    print $text, "\n";
    print "</div>\n";

    $first = 0;

}


if ( $first ){ # no records found
    print "<p style=\"font-size: 150%\"><b>",$lex{'No Records Found'},"</b>.</p>\n";
} else {
    print "</table>\n";
    print "[ <a href=\"#top\">$lex{'Top'}</a> ]\n";
}

print "</body></html>\n";



#----------------
sub showStartPage {
#----------------

    # print sortorder and selection input form.
    print "<h1>". $lex{View}. q{ }. $lex{Students}. "</h1>\n";
    print "<div><form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";
    if ( $arr{table} ) { # for withdrawn mode.
	print "<input type=\"hidden\" name=\"table\" value=\"wd\">\n";
    }

    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\">\n";

    print "<tr><td align=\"right\">". $lex{'Sort by'}. "\n";
    print "</td><td><select name=\"sortorder\"><option>". $lex{'Lastname'}. "</option>\n";
    print "<option>". $lex{'Birthdate'}. "</option>";
    print "</select></td></tr>\n";

    print "<tr><td align=\"right\">". $lex{'Select by'}. "\n";
    print "</td><td><select name=\"groupType\"><option>". $lex{Grade}. "</option>\n";
    print "<option>". $lex{Homeroom}. "</option>\n";
    print "</select>\n";
    print "<input type=\"text\" name=\"groupValue\" size=\"10\"></td></tr>\n";
    
    print "<tr><td></td><td align=\"left\">\n";
    print "<input type=\"submit\" value=\"". $lex{Continue}. "\"></td></tr>\n";


    print "</table></form></div>\n";
    print "</body></html>\n";

    exit 0;


}
